{"filter":false,"title":"README.md","tooltip":"/sources/database/README.md","undoManager":{"mark":0,"position":0,"stack":[[{"start":{"row":0,"column":0},"end":{"row":257,"column":31},"action":"insert","lines":["# Memored","","Memored implements an in-memory shared cache to use in nodejs applications which uses [cluster](http://nodejs.org/api/cluster.html) module.","","Let's say you want your application to take advantage of multi-core CPUs using nodejs cluster module; you will be able to run several _isolated_ processes which shared nothing but a communication channel with parent process.","If you need a fast volatile cache, common solutions would create an in-memory map for every process you run, so you end up with the same data stored several times in your machine RAM.","","Memored uses communication channel between master process and its workers to use a unique in-memory storage, reducing the amount of memory your application would use.","","## Getting Started","Install this module with _npm_:","```javascript","npm install memored","```","","Store and read values is straightforward:","```javascript","var cluster = require('cluster'),","\tmemored = require('memored');","","if (cluster.isMaster) {","\tcluster.fork();","} else {","\tvar han = {","\t\t\tfirstname: 'Han',","\t\t\tlastname: 'Solo'","\t\t},","\t\tluke = {","\t\t\tfirstname: 'Luke',","\t\t\tlastname: 'Skywalker'","\t\t};","","\t// Store and read","\tmemored.store('character1', han, function() {","\t\tconsole.log('Value stored!');","","\t\tmemored.read('character1', function(err, value) {","\t\t\tconsole.log('Read value:', value);","\t\t});","\t});","","\t// You can also set a ttl (milliseconds)","\tmemored.store('character2', luke, 1000, function(err, expirationTime) {","\t\tconsole.log('Value stored until:', new Date(expirationTime));","","\t\tsetTimeout(function() {","\t\t\tmemored.read('character2', function(err, value) {","\t\t\t\tconsole.log('Value is gone?', value === undefined);","","\t\t\t\tprocess.exit();","\t\t\t});","\t\t}, 1050);","\t});","}","```","","## Invalidation management","By default, _memored_ will evict cache entries (stored with _ttl_) passively. This is, when you read an expired entry, you will get no value on return and _memored_ will delete the value from its internal cache.","","You can also configure _memored_ to actively evict expired entries every N milliseconds. For this to work, you need to pass the attribute *purgeInterval* to the *setup* function.","This will trigger an internal function which looks for expired entries and deletes them from its internal cache.","","Example:","```javascript","var cluster = require('cluster'),","\tasync = require('async'),","\tmemored = require('memored');","","if (cluster.isMaster) {","","\tcluster.fork();","\tmemored.setup({ purgeInterval: 500});","","} else {","","\tasync.series({","\t\tstoreValue: function(next) {","\t\t\tmemored.store('key1', 'My simple string value', 100, next);","\t\t},","\t\treadCacheSize: function(next) {","\t\t\tmemored.size(function(err, size) {","\t\t\t\tconsole.log('Current size is 1?', size === 1);","\t\t\t\tnext();","\t\t\t});","\t\t},","\t\twait: function(next) {","\t\t\tsetTimeout(next, 600);","\t\t},","\t\treadCacheSizeAgain: function(next) {","\t\t\tmemored.size(function(err, size) {","\t\t\t\tconsole.log('Current size is 0?', size === 0);","\t\t\t\tnext();","\t\t\t});","\t\t}","\t}, process.exit);","}","```","","## API","Documentation for every module function:","","","### setup(options)","","This function is used to configure _memored_.","","**Arguments**:","","- **purgeInterval** {Number} (optional): Configures and triggers _memored_ expired entries auto deletion. Value expressed in milliseconds. _It's only used when called this method from the master process of your application_.","- **logger** {Object} (optional): In you want _memored_ to log something, you must provide an object implementing *log* and *warn* functions.","","**Example**:","```javascript","memored.setup({","\tpurgeInterval: 15000,","\tlogger: console","});","```","","### store(key, value, [ttl], callback)","","This function stores a value in the cache.","_It is intended to be called from a worker process_.","","**Arguments**:","","- **key** {String} (required): Key used to lookup the entry","- **value** {Mixed} (required): Whatever you want to store","- **ttl** {Number} (optional): Time to live for this value in the cache (milliseconds)","- **callback** {Function} (optional): Function to be call on store completion. Callback arguments:","\t- _err_ {Error}: Optional error","\t- _expirationTime_ {Number}: The timestamp of the moment when this entry will expire. If _ttl_ is not used, this value will be undefined.","","**Examples**:","```javascript","memored.store('key1', {firstname: 'Han', lastname: 'Solo'}, function() {","\tconsole.log('Value stored!');","});","","memored.store('key2', ['a', 'b', 'c'], 15000, function(err, expirationTime) {","\tconsole.log('This value will expire on:', new Date(expirationTime));","});","```","","### read(key, callback)","","This function reads a value from the cache.","_It is intended to be called from a worker process_.","","**Arguments**:","","- **key** {String} (required): Key used to lookup the entry","- **callback** {Function} (required): Function to be called on read completion. Callback arguments:","\t- _err_ {Error}: Optional error","\t- _value_ {Mixed}: Contents of the cached entry. If the value is not found or is expired, it will be undefined.","","**Example**:","```javascript","memored.read('key1', function(err, value) {","\tconsole.log('Key1 value:', value);","});","","memored.read('key1', function(err, value, expirationTime) {","\tconsole.log('Key1 value:', value);","\tconsole.log('Key1 expiration time:', new Date(expirationTime));","});","","memored.read('unknownKey', function(err, value) {","\tconsole.log('No data read?', value === undefined);","});","```","","### remove(key, callback)","","This function removes an entry from the cache.","_It is intended to be called from a worker process_.","","**Arguments**:","","- **key** {String} (required): Key used to lookup the entry","- **callback** {Function} (optional): Function to be called on read completion.","","**Example**:","```javascript","memored.remove('key1', function() {","\tconsole.log('Key removed from the cache.');","});","```","","### clean(callback)","","This function removes all the entries from the cache.","_It is intended to be called from a worker process_.","","**Arguments**:","","- **callback** {Function} (optional): Function to be called on read completion.","","**Example**:","```javascript","memored.clean(function() {","\tconsole.log('All cache entries have been deleted.');","});","```","","### size(callback)","","This function returns the number of entries in the cache.","","**Arguments**:","","- **callback** {Function} (required): Function to be called on size calculation is complete. Callback arguments:","\t- _err_ {Error}: Optional error","\t- _size_ {Number}: The number of entries in the cache.","","**Example**:","","```javascript","memored.size(function(err, size) {","\tconsole.log('Cache size:', size);","});","```","","### keys(callback)","","This function returns an array of the keys for objects in the cache.","","**Arguments**:","","- **callback** {Function} (required): Function to be called when keys calculation is complete. Callback arguments:","\t- _err_ {Error}: Optional error","\t- _keys_ {Array}: An array of strings for the keys of the entries in the cache.","","**Example**:","","```javascript","memored.keys(function(err, keys) {","\tconsole.log('Cache keys:', keys);","});","```","","### version","","This is an attribute which provides module's version number","","","# Final note","All the callbacks first parameter is an optional error object. Actually, this param will never be an error because there is no expected error in the internal code. There's no function call that can possible throw an expected error that this module would deal with.","The existence of this param is to follow the *convention* about libraries callbacks in nodejs. As everybody expects this first callback parameter to be an optional one, I decided to include it.","","- [Nodeguide](http://nodeguide.com/style.html#callbacks)","- [Nodejitsu](http://docs.nodejitsu.com/articles/errors/what-are-the-error-conventions)","- [GoSquared](https://engineering.gosquared.com/node-js-error-handling-callbacks-vs-promises)","","","## License","Copyright (c) 2014 PaquitoSoft  ","Licensed under the MIT license."],"id":1}]]},"ace":{"folds":[],"scrolltop":3154,"scrollleft":0,"selection":{"start":{"row":257,"column":31},"end":{"row":257,"column":31},"isBackwards":false},"options":{"guessTabSize":true,"useWrapMode":false,"wrapToView":true},"firstLineState":{"row":209,"state":"start","mode":"ace/mode/markdown"}},"timestamp":1469556776572,"hash":"b0b5c7fb3a3f2c911b7bfa0cb7d2c99938d4a4a2"}